/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2014 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE TYPE BODY xxas_http_req_obj IS
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_request_xml
  -- ---------------------------------------------------------------------- * 
  MEMBER FUNCTION get_request_xml RETURN XMLType
  IS
  
  BEGIN
  
    RETURN req;
    
  END get_request_xml;

  -- ---------------------------------------------------------------------- *
  -- Name    : get_header
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_header
    (p_header_name IN varchar2) RETURN varchar2
  IS

  BEGIN
  
    RETURN xxas_util_pkg.extract_string
             (req
             ,'r/rh/h[@n = "' || upper(p_header_name) || '"]/v[1]/text()');
    
  END get_header;

  -- ---------------------------------------------------------------------- *
  -- Name    : get_parameters
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_parameters RETURN xxas_com_srv_param_varray
  IS

    l_parameters xxas_com_srv_param_varray;
  
  BEGIN
  
    SELECT cast
             (multiset
               (SELECT xxas_com_srv_param_value(p.parameter_name, p.parameter_value)
                FROM  XMLTable
                       ('r/rp/p'
                       PASSING req
                       COLUMNS parameter_name  VARCHAR2(4000) path '@n'
                              ,parameter_value VARCHAR2(4000) path 'v/text()') p)
              AS xxas_com_srv_param_varray)
    INTO  l_parameters
    FROM  dual;
    
    RETURN l_parameters;
    
  END get_parameters;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_parameter
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_parameter
    (p_parameter_name IN varchar2) RETURN varchar2
  IS

  BEGIN
  
    RETURN xxas_util_pkg.extract_string
             (req, 'r/rp/p[@n = "' || p_parameter_name || '"]/v[1]/text()');
    
  END get_parameter;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_content_type
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_content_type RETURN varchar2
  IS

  BEGIN
  
    RETURN get_header('CONTENT-TYPE');
    
  END get_content_type;

  -- ---------------------------------------------------------------------- *
  -- Name    : get_request_body
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_request_body RETURN CLOB
  IS

    l_clob CLOB;
  
  BEGIN
  
    l_clob := xxas_util_pkg.extract_clob(req, 'r/b[1]/text()');
    IF l_clob IS NULL THEN
      RETURN xxas_util_pkg.extract_clob
               (req, 'r/rp/p[@n = "_entity"]/v[1]/text()');
    END IF;
    RETURN l_clob;
    
  END get_request_body;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_request_body_xml
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_request_body_xml RETURN XMLType
  IS

  BEGIN
  
    RETURN XMLType(get_request_body);
    
  END get_request_body_xml;

  -- ---------------------------------------------------------------------- *
  -- Name    : is_post
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION is_post
    (p_method IN varchar2) RETURN boolean
  IS

  BEGIN

    RETURN (upper(p_method) = 'POST');
    
  END is_post;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : is_valid_method
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION is_valid_method
    (p_method IN varchar2) RETURN boolean
  IS

  BEGIN

    RETURN (upper(p_method) = 'GET'     OR
            upper(p_method) = 'POST'    OR
            upper(p_method) = 'HEAD'    OR
            upper(p_method) = 'OPTIONS' OR
            upper(p_method) = 'PUT'     OR
            upper(p_method) = 'DELETE'  OR
            upper(p_method) = 'TRACE'   OR
            upper(p_method) = 'CONNECT');
    
  END is_valid_method;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_method
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_method RETURN varchar2
  IS

    l_method        varchar2(30);
    l_override      varchar2(30);

  BEGIN

    --
    -- Many HTTP clients/servers do not support the PUT and DELETE
    -- verbs.  Allow the HTTP method to be overridden using the following
    -- rules:
    --
    --   1) If the _method form parameter is specified use that,
    --   2) If the X-HTTP-Method-Override header is specified use that
    --
    l_method := substr
                  (upper
                    (xxas_util_pkg.extract_string(req, 'r/@method'))
                  ,1, 30);
    
    IF is_post(l_method) THEN
    
      --
      -- First get the override method from the _method form parameter
      --
      l_override := substr(upper(get_parameter('_method')), 1, 30);
      IF NOT is_valid_method(l_override) THEN
        l_override := NULL;
      END IF;
      
      --
      -- Get the override method from the X-HTTP-Method-Override
      -- header if specified
      --
      IF l_override IS NULL THEN
        l_override := substr
                        (upper
                          (get_header('X-HTTP-METHOD-OVERRIDE'))
                        ,1, 30);
        IF NOT is_valid_method(l_override) THEN
          l_override := NULL;
        END IF;
      END IF;
      
      --
      -- Set the override header if valid
      --
      IF l_override IS NOT NULL THEN
        l_method := l_override;
      END IF;
    END IF;
    RETURN l_method;
    
  END get_method;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_accept
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_accept RETURN varchar2
  IS

    l_accept        varchar2(400);
    l_override      varchar2(400);

  BEGIN

    --
    -- Some clients (eg, Flash) strip the browser of headers, so this allows
    -- the Accept to be specified as a request parameter.
    --
    l_override := substr(get_parameter('_accept'), 1, 400);
    IF l_override IS NOT NULL THEN
      RETURN l_override;
    ELSE
      RETURN get_header('Accept');
    END IF;
    
  END get_accept;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_path_info
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_path_info RETURN varchar2
  IS

    l_path          varchar2(32767);
    l_instr         number;

  BEGIN
  
    l_path  := xxas_util_pkg.extract_string(req, 'r/@path');
    l_instr := instr(l_path, 'index.htm', -1, 1);
    IF l_instr > 0 THEN
      l_path :=  substr(l_path, 1, l_instr - 1);
    END IF;
    RETURN l_path;
    
  END get_path_info;

  -- ---------------------------------------------------------------------- *
  -- Name    : get_context_path
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_context_path RETURN varchar2
  IS

  BEGIN
  
    RETURN xxas_util_pkg.extract_string(req, 'r/@ctxPath');
    
  END get_context_path;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_host
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_host RETURN varchar2
  IS

  BEGIN
  
    RETURN xxas_util_pkg.extract_string(req, 'r/@url');
    
  END get_host;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_full_url
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_full_url RETURN varchar2
  IS

  BEGIN
  
    RETURN xxas_util_pkg.extract_string(req, 'r/@fullURL');
    
  END get_full_url;
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_scheme
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_scheme RETURN varchar2
  IS

    l_PROC CONSTANT varchar2(30) := 'get_scheme';

  BEGIN
  
    RETURN xxas_util_pkg.extract_string(req, 'r/@scheme');
    
  END get_scheme;
  
  
  -- ---------------------------------------------------------------------- *
  -- Name    : get_remote_addr
  -- ---------------------------------------------------------------------- *   
  MEMBER FUNCTION get_remote_addr RETURN varchar2
  IS

    l_PROC CONSTANT varchar2(30) := 'get_remote_addr';

  BEGIN
  
    RETURN xxas_util_pkg.extract_string(req, 'r/@remoteAddr');
    
  END get_remote_addr;

END;
/

COMMIT;
EXIT;
